package MooseX::AttributeHelpers::MethodProvider::Counter;
use MooseX::AttributeHelpers::MethodProvider;

our $VERSION   = '0.02';
our $AUTHORITY = 'cpan:STEVAN';

add_method_provider 'Counter' => (
    type => 'Int',
    provides => {
        reset => sub {
            my ($attr, $reader, $writer) = @_;
            return sub { $writer->($_[0], $attr->default($_[0])) };
        },

        inc => sub {
            my ($attr, $reader, $writer) = @_;
            return sub { $writer->($_[0], $reader->($_[0]) + 1) };
        },

        dec => sub {
            my ($attr, $reader, $writer) = @_;
            return sub { $writer->($_[0], $reader->($_[0]) - 1) };
        },
    },
);

1;

__END__

=pod

=head1 NAME

MooseX::AttributeHelpers::MethodProvider::Counter

=head1 DESCRIPTION

This module provides the method factories for 
L<MooseX::AttributeHelpers::Counter>.

=head1 PROVIDED METHODS

=over 4

=item I<inc>

Increments the value stored in this slot by 1.

=item I<dec>

Decrements the value stored in this slot by 1.

=item I<reset>

Resets the value stored in this slot to its default value.

=back

=head1 BUGS

All complex software has bugs lurking in it, and this module is no
exception. If you find a bug please either email me, or add the bug
to cpan-RT.

=head1 AUTHOR

Stevan Little E<lt>stevan@iinteractive.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2007-2008 by Infinity Interactive, Inc.

L<http://www.iinteractive.com>

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
